@@ -4,20 +4,19 @@
 #include "config.h"
 #include "cstring.h"
 
-/* reference-style links (and images) are stored in an array
- * of footnotes.
+#ifdef HAVE_INTTYPES_H
+#   include <inttypes.h>
+#elif HAVE_STDINT_H
+#   include <stdint.h>
+#endif
+
+/* flags, captured into a named type
  */
-typedef struct footnote {
-    Cstring tag;		/* the tag for the reference link */
-    Cstring link;		/* what this footnote points to */
-    Cstring title;		/* what it's called (TITLE= attribute) */
-    int height, width;		/* dimensions (for image link) */
-    int dealloc;		/* deallocation needed? */
-    int refnumber;
-    int flags;
-#define EXTRA_BOOKMARK	0x01
-#define REFERENCED	0x02
-} Footnote;
+typedef DWORD mkd_flag_t;
+
+#define is_flag_set(flags, item)	((flags) & (item))
+#define set_flag(flags, item)		((flags) |= (item))
+#define clear_flag(flags, item)		((flags) &= ~(item))
 
 /* each input line is read into a Line, which contains the line,
  * the offset of the first non-space character [this assumes 
@@ -56,10 +55,32 @@ typedef struct paragraph {
 	   HDR, HR, TABLE, SOURCE } typ;
     enum { IMPLICIT=0, PARA, CENTER} align;
     int hnumber;		/* <Hn> for typ == HDR */
+#if GITHUB_CHECKBOX
+    int flags;
+#define GITHUB_CHECK		0x01
+#define IS_CHECKED		0x02
+#endif
 } Paragraph;
 
 enum { ETX, SETEXT };	/* header types */
 
+/* reference-style links (and images) are stored in an array
+ * of footnotes.
+ */
+typedef struct footnote {
+    Cstring tag;		/* the tag for the reference link */
+    Cstring link;		/* what this footnote points to */
+    Cstring title;		/* what it's called (TITLE= attribute) */
+    Paragraph *text;		/* EXTRA_FOOTNOTE content */
+    
+    int height, width;		/* dimensions (for image link) */
+    int dealloc;		/* deallocation needed? */
+    int refnumber;
+    int flags;
+#define EXTRA_FOOTNOTE	0x01
+#define REFERENCED	0x02
+} Footnote;
+
 
 typedef struct block {
     enum { bTEXT, bSTAR, bUNDER } b_type;
@@ -79,7 +100,9 @@ typedef struct callback_data {
     void *e_data;		/* private data for callbacks */
     mkd_callback_t e_url;	/* url edit callback */
     mkd_callback_t e_flags;	/* extra href flags callback */
+    mkd_callback_t e_anchor;	/* callback for anchor types */
     mkd_free_t e_free;		/* edit/flags callback memory deallocator */
+    mkd_callback_t e_codefmt;	/* codeblock formatter (for highlighting) */
 } Callback_data;
 
 
@@ -102,11 +125,12 @@ typedef struct mmiot {
     Cstring out;
     Cstring in;
     Qblock Q;
+    char last;	/* last text character added to out */
     int isp;
     struct escaped *esc;
     char *ref_prefix;
     struct footnote_list *footnotes;
-    DWORD flags;
+    mkd_flag_t flags;
 #define MKD_NOLINKS	0x00000001
 #define MKD_NOIMAGE	0x00000002
 #define MKD_NOPANTS	0x00000004
@@ -138,7 +162,8 @@ typedef struct mmiot {
 #define MKD_URLENCODEDANCHOR 0x10000000
 #define IS_LABEL	0x20000000
 #define MKD_LATEX	0x40000000
-#define USER_FLAGS	0x7FFFFFFF
+#define MKD_EXPLICITLIST	0x80000000
+#define USER_FLAGS	0xFFFFFFFF
 #define INPUT_MASK	(MKD_NOHEADER|MKD_TABSTOP)
 
     Callback_data *cb;
@@ -164,6 +189,7 @@ typedef struct document {
     ANCHOR(Line) content;	/* uncompiled text, not valid after compile() */
     Paragraph *code;		/* intermediate code generated by compile() */
     int compiled;		/* set after mkd_compile() */
+    int dirty;			/* flags or callbacks changed */
     int html;			/* set after (internal) htmlify() */
     int tabstop;		/* for properly expanding tabs (ick) */
     char *ref_prefix;
@@ -183,7 +209,7 @@ struct string_stream {
 
 
 extern int  mkd_firstnonblank(Line *);
-extern int  mkd_compile(Document *, DWORD);
+extern int  mkd_compile(Document *, mkd_flag_t);
 extern int  mkd_document(Document *, char **);
 extern int  mkd_generatehtml(Document *, FILE *);
 extern int  mkd_css(Document *, char **);
@@ -192,19 +218,19 @@ extern int  mkd_generatecss(Document *, FILE *);
 extern int  mkd_xml(char *, int , char **);
 extern int  mkd_generatexml(char *, int, FILE *);
 extern void mkd_cleanup(Document *);
-extern int  mkd_line(char *, int, char **, DWORD);
-extern int  mkd_generateline(char *, int, FILE*, DWORD);
+extern int  mkd_line(char *, int, char **, mkd_flag_t);
+extern int  mkd_generateline(char *, int, FILE*, mkd_flag_t);
 #define mkd_text mkd_generateline
 extern void mkd_basename(Document*, char *);
 
 typedef int (*mkd_sta_function_t)(const int,const void*);
-extern void mkd_string_to_anchor(char*,int, mkd_sta_function_t, void*, int, DWORD);
+extern void mkd_string_to_anchor(char*,int, mkd_sta_function_t, void*, int, MMIOT *);
 
-extern Document *mkd_in(FILE *, DWORD);
-extern Document *mkd_string(const char*,int, DWORD);
+extern Document *mkd_in(FILE *, mkd_flag_t);
+extern Document *mkd_string(const char*, int, mkd_flag_t);
 
-extern Document *gfm_in(FILE *, DWORD);
-extern Document *gfm_string(const char*,int, DWORD);
+extern Document *gfm_in(FILE *, mkd_flag_t);
+extern Document *gfm_string(const char*,int, mkd_flag_t);
 
 extern void mkd_initialize();
 extern void mkd_shlib_destructor();
@@ -222,13 +248,13 @@ extern void ___mkd_initmmiot(MMIOT *, void *);
 extern void ___mkd_freemmiot(MMIOT *, void *);
 extern void ___mkd_freeLineRange(Line *, Line *);
 extern void ___mkd_xml(char *, int, FILE *);
-extern void ___mkd_reparse(char *, int, int, MMIOT*, char*);
+extern void ___mkd_reparse(char *, int, mkd_flag_t, MMIOT*, char*);
 extern void ___mkd_emblock(MMIOT*);
 extern void ___mkd_tidy(Cstring *);
 
 extern Document *__mkd_new_Document();
 extern void __mkd_enqueue(Document*, Cstring *);
-extern void __mkd_header_dle(Line *);
+extern void __mkd_trim_line(Line *, int);
 
 extern int  __mkd_io_strget(struct string_stream *);
 
